@@ -1,26 +1,29 @@
 package Code::TidyAll::Cache;
 
+use strict;
+use warnings;
+
 use Digest::SHA qw(sha1_hex);
-use Code::TidyAll::Util qw(dirname mkpath);
-use File::Slurp::Tiny qw(read_file write_file);
+use Path::Tiny qw(path);
+
 use Moo;
 
-our $VERSION = '0.32';
+our $VERSION = '0.55';
 
 has 'cache_dir' => ( is => 'ro', required => 1 );
 
 sub path_to_key {
     my ( $self, $key ) = @_;
     my $sig = sha1_hex($key);
-    return join( "/", $self->cache_dir, substr( $sig, 0, 1 ), "$sig.dat" );
+    return $self->cache_dir->child( substr( $sig, 0, 1 ), "$sig.dat" );
 }
 
 sub get {
     my ( $self, $key ) = @_;
 
     my $file = $self->path_to_key($key);
-    if ( defined $file && -f $file ) {
-        return read_file($file);
+    if ( $file->exists ) {
+        return $file->slurp;
     }
     else {
         return undef;
@@ -31,15 +34,18 @@ sub set {
     my ( $self, $key, $value ) = @_;
 
     my $file = $self->path_to_key($key);
-    mkpath( dirname($file), 0, 0775 );
-    write_file( $file, $value );
+    $file->parent->mkpath( { mode => 0755 } );
+    $file->spew($value);
+
+    return;
 }
 
 sub remove {
     my ( $self, $key, $value ) = @_;
 
-    my $file = $self->path_to_key($key);
-    unlink($file);
+    $self->path_to_key($key)->remove;
+
+    return;
 }
 
 1;
