@@ -4,9 +4,10 @@ use strict;
 use warnings;
 
 use JSON::MaybeXS ();
+
 use Moo;
 
-our $VERSION = '0.32';
+our $VERSION = '0.55';
 
 extends 'Code::TidyAll::Plugin';
 
@@ -36,6 +37,8 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Code::TidyAll::Plugin::JSON - Use the JSON::MaybeXS module to tidy JSON
@@ -43,7 +46,7 @@ documents with tidyall
 
 =head1 VERSION
 
-version 0.32
+version 0.55
 
 =head1 SYNOPSIS
 
@@ -68,9 +71,12 @@ Escape non-ASCII characters. The output file will be valid ASCII.
 
 =back
 
-=head1 SEE ALSO
+=head1 SUPPORT
 
-L<Code::TidyAll>
+Bugs may be submitted through
+L<https://github.com/houseabsolute/perl-code-tidyall/issues>.
+
+I am also usually active on IRC as 'drolsky' on C<irc://irc.perl.org>.
 
 =head1 AUTHORS
 
@@ -88,7 +94,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 - 2015 by Jonathan Swartz.
+This software is copyright (c) 2011 - 2016 by Jonathan Swartz.
 
 This is free software; you can redistribute it and/or modify it under the same
 terms as the Perl 5 programming language system itself.
