@@ -1,11 +1,16 @@
 package Code::TidyAll::Plugin::PodTidy;
 
+use strict;
+use warnings;
+
 use Capture::Tiny qw(capture_merged);
 use Pod::Tidy;
+
 use Moo;
+
 extends 'Code::TidyAll::Plugin';
 
-our $VERSION = '0.32';
+our $VERSION = '0.55';
 
 has 'columns' => ( is => 'ro' );
 
@@ -14,7 +19,7 @@ sub transform_file {
 
     my $output = capture_merged {
         Pod::Tidy::tidy_files(
-            files    => [$file],
+            files    => [ $file->stringify ],
             inplace  => 1,
             nobackup => 1,
             verbose  => 1,
@@ -32,13 +37,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Code::TidyAll::Plugin::PodTidy - Use podtidy with tidyall
 
 =head1 VERSION
 
-version 0.32
+version 0.55
 
 =head1 SYNOPSIS
 
@@ -68,9 +75,12 @@ Number of columns per line
 
 =back
 
-=head1 SEE ALSO
+=head1 SUPPORT
 
-L<Code::TidyAll>
+Bugs may be submitted through
+L<https://github.com/houseabsolute/perl-code-tidyall/issues>.
+
+I am also usually active on IRC as 'drolsky' on C<irc://irc.perl.org>.
 
 =head1 AUTHORS
 
@@ -88,7 +98,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 - 2015 by Jonathan Swartz.
+This software is copyright (c) 2011 - 2016 by Jonathan Swartz.
 
 This is free software; you can redistribute it and/or modify it under the same
 terms as the Perl 5 programming language system itself.
