@@ -1,23 +1,22 @@
 package Code::TidyAll::Role::Tempdir;
 
+use strict;
+use warnings;
+
 use Moo::Role;
 
-use Cwd qw(realpath);
-use File::Temp qw(tempdir);
+use Path::Tiny qw(tempdir);
 
-our $VERSION = '0.32';
+our $VERSION = '0.55';
 
 has '_tempdir'   => ( is => 'ro', lazy => 1, builder => 1 );
 has 'no_cleanup' => ( is => 'ro', default => 0 );
 
 sub _build__tempdir {
     my ($self) = @_;
-    return realpath(
-        tempdir(
-            'Code-TidyAll-XXXX',
-            TMPDIR  => 1,
-            CLEANUP => !$self->no_cleanup,
-        )
+    return tempdir(
+        'Code-TidyAll-XXXX',
+        CLEANUP => !$self->no_cleanup,
     );
 }
 
@@ -29,6 +28,8 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Code::TidyAll::Role::Tempdir - Provides a _tempdir attribute for Code::TidyAll
@@ -36,7 +37,7 @@ classes
 
 =head1 VERSION
 
-version 0.32
+version 0.55
 
 =head1 SYNOPSIS
 
@@ -48,8 +49,6 @@ version 0.32
 
 A role to add tempdir attributes to classes.
 
-=encoding UTF-8
-
 =head1 ATTRIBUTES
 
 =over
@@ -65,9 +64,12 @@ should not automatically clean up after itself
 
 =back
 
-=head1 SEE ALSO
+=head1 SUPPORT
 
-L<Code::TidyAll>
+Bugs may be submitted through
+L<https://github.com/houseabsolute/perl-code-tidyall/issues>.
+
+I am also usually active on IRC as 'drolsky' on C<irc://irc.perl.org>.
 
 =head1 AUTHORS
 
@@ -85,7 +87,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 - 2015 by Jonathan Swartz.
+This software is copyright (c) 2011 - 2016 by Jonathan Swartz.
 
 This is free software; you can redistribute it and/or modify it under the same
 terms as the Perl 5 programming language system itself.
