@@ -1,12 +1,14 @@
 package Code::TidyAll::SVN::Util;
 
+use strict;
+use warnings;
+
 use Cwd qw(realpath);
 use IPC::System::Simple qw(capturex);
-use strict;
-use warnings;
-use base qw(Exporter);
 
-our $VERSION = '0.32';
+use Exporter qw(import);
+
+our $VERSION = '0.55';
 
 our @EXPORT_OK = qw(svn_uncommitted_files);
 
