@@ -1,14 +1,19 @@
 package Test::Code::TidyAll;
 
-use Code::TidyAll;
-use Test::Builder;
-use Text::Diff;
-use Text::Diff::Table;
 use strict;
 use warnings;
-use base qw(Exporter);
 
-our $VERSION = '0.32';
+use Code::TidyAll;
+use List::Compare;
+use Test::Builder;
+
+# Text::Diff has to be loaded before ::Table
+use Text::Diff;
+use Text::Diff::Table;
+
+use Exporter qw(import);
+
+our $VERSION = '0.55';
 
 my $test = Test::Builder->new;
 
@@ -25,23 +30,36 @@ sub tidyall_ok {
     $options{quiet} = 1 unless $options{verbose};
     $test->diag("Using $conf_file for config")
         if $options{verbose};
-    my $ct = Code::TidyAll->new_from_conf_file(
+
+    my $files = delete $options{files};
+    my $ct    = Code::TidyAll->new_from_conf_file(
         $conf_file,
         check_only    => 1,
         mode          => 'test',
         msg_outputter => \&_msg_outputter,
         %options,
     );
-    my @files = $ct->find_matched_files;
+
+    my @files;
+    if ($files) {
+        @files = List::Compare->new( $files, [ $ct->find_matched_files ] )->get_intersection;
+    }
+    else {
+        @files = $ct->find_matched_files;
+    }
+
+    unless (@files) {
+        $test->plan( tests => 1 );
+        $test->ok( 1, 'found no matching files for tidyall_ok' );
+        return;
+    }
+
     $test->plan( tests => scalar(@files) );
     foreach my $file (@files) {
         my $desc   = $ct->_small_path($file);
         my $result = $ct->process_file($file);
-        if ( $result->ok ) {
-            $test->ok( 1, $desc );
-        }
-        else {
-            $test->ok( 0, $desc );
+        $test->ok( $result->ok, "$desc is tidy" );
+        unless ( $result->ok ) {
             $test->diag( $result->error );
 
             if ( $options{verbose} ) {
@@ -68,6 +86,8 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Test::Code::TidyAll - Check that all your files are tidy and valid according to
@@ -75,7 +95,7 @@ tidyall
 
 =head1 VERSION
 
-version 0.32
+version 0.55
 
 =head1 SYNOPSIS
 
@@ -87,30 +107,50 @@ version 0.32
 
 =head1 DESCRIPTION
 
-Uses L<tidyall --check-only|tidyall> to check that all the files in your
+Uses L<Code::TidyAll>'s C<check_only> mode to check that all the files in your
 project are in a tidied and valid state, i.e. that no plugins throw errors or
 would change the contents of the file. Does not actually modify any files.
 
-By default, looks for config file C<tidyall.ini> or C<.tidyallrc> in the
+By default, we look for the config file C<tidyall.ini> or C<.tidyallrc> in the
 current directory and parent directories, which is generally the right place if
 you are running L<prove>.
 
-Passes mode = "test" by default; see L<modes|tidyall/MODES>.
+When invoking L<Code::TidyAll>, we pass C<<mode => 'test'>> by default; see
+L<modes|tidyall/MODES>.
 
-C<tidyall_ok> is exported by default. Any options will be passed along to the
+=head1 EXPORTS
+
+This module exports one subroutine, which is exported by default:
+
+=head2 tidyall_ok(...)
+
+Most options given to this subroutine will be passed along to the
 L<Code::TidyAll> constructor. For example, if you don't want to use the tidyall
 cache and instead check all files every time:
 
-    tidyall_ok(no_cache => 1);
+    tidyall_ok( no_cache => 1 );
 
 or if you need to specify the config file:
 
-    tidyall_ok(conf_file => '/path/to/conf/file');
+    tidyall_ok( conf_file => '/path/to/conf/file' );
+
+By default, this subroutine will test every file that matches the config you
+specify. However, you can pass a C<files> parameter as an array reference to
+override this, in which case only the files you specify will be tested. These
+files are still filtered based on the C<select> and C<exclude> rules defined in
+your config.
 
 =head1 SEE ALSO
 
 L<tidyall>
 
+=head1 SUPPORT
+
+Bugs may be submitted through
+L<https://github.com/houseabsolute/perl-code-tidyall/issues>.
+
+I am also usually active on IRC as 'drolsky' on C<irc://irc.perl.org>.
+
 =head1 AUTHORS
 
 =over 4
@@ -127,7 +167,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 - 2015 by Jonathan Swartz.
+This software is copyright (c) 2011 - 2016 by Jonathan Swartz.
 
 This is free software; you can redistribute it and/or modify it under the same
 terms as the Perl 5 programming language system itself.
