@@ -1,13 +1,13 @@
 package Test::Code::TidyAll::Basic;
 
-use Cwd qw(realpath);
-use Code::TidyAll::Util qw(dirname mkpath pushd tempdir_simple);
-use Code::TidyAll;
 use Capture::Tiny qw(capture capture_stdout capture_merged);
-use File::Find qw(find);
-use File::Slurp::Tiny qw(read_file write_file);
-use Test::Class::Most parent => 'Code::TidyAll::Test::Class';
 use Code::TidyAll::CacheModel::Shared;
+use Code::TidyAll::Util qw(pushd tempdir_simple);
+use Code::TidyAll;
+use File::Find qw(find);
+use Path::Tiny qw(cwd path);
+
+use Test::Class::Most parent => 'Code::TidyAll::Test::Class';
 
 sub test_plugin {"+Code::TidyAll::Test::Plugin::$_[0]"}
 my %UpperText  = ( test_plugin('UpperText')  => { select => '**/*.txt' } );
@@ -16,53 +16,17 @@ my %RepeatFoo  = ( test_plugin('RepeatFoo')  => { select => '**/foo*' } );
 my %CheckUpper = ( test_plugin('CheckUpper') => { select => '**/*.txt' } );
 my %AToZ       = ( test_plugin('AToZ')       => { select => '**/*.txt' } );
 
-my $cli_conf;
+my $cli_conf = <<'EOF';
+backup_ttl = 15m
+verbose = 1
 
-sub create_dir {
-    my ( $self, $files ) = @_;
+[+Code::TidyAll::Test::Plugin::UpperText]
+select = **/*.txt
 
-    my $root_dir = tempdir_simple();
-    while ( my ( $path, $content ) = each(%$files) ) {
-        my $full_path = "$root_dir/$path";
-        mkpath( dirname($full_path), 0, 0775 );
-        write_file( $full_path, $content );
-    }
-    return realpath($root_dir);
-}
-
-sub tidy {
-    my ( $self, %params ) = @_;
-    my $desc = $params{desc};
-    if ( !defined($desc) ) {
-        ($desc) = ( ( caller(1) )[3] =~ /([^:]+$)/ );
-    }
-
-    my $root_dir = $self->create_dir( $params{source} );
-
-    my $options = $params{options} || {};
-    my $ct = Code::TidyAll->new(
-        plugins  => $params{plugins},
-        root_dir => $root_dir,
-        %$options
-    );
-
-    my @results;
-    my $output = capture_stdout { @results = $ct->process_all() };
-    my $error_count = grep { $_->error } @results;
-    if ( $params{errors} ) {
-        like( $output, $params{errors}, "$desc - errors" );
-        ok( $error_count > 0, "$desc - error_count > 0" );
-    }
-    else {
-        is( $error_count, 0, "$desc - error_count == 0" );
-    }
-    while ( my ( $path, $content ) = each( %{ $params{dest} } ) ) {
-        is( read_file("$root_dir/$path"), $content, "$desc - $path content" );
-    }
-    if ( my $like_output = $params{like_output} ) {
-        like( $output, $like_output, "$desc - output" );
-    }
-}
+[+Code::TidyAll::Test::Plugin::RepeatFoo]
+select = **/foo*
+times = 3
+EOF
 
 sub test_basic : Tests {
     my $self = shift;
@@ -91,7 +55,7 @@ sub test_basic : Tests {
     );
     $self->tidy(
         plugins => { %UpperText, %ReverseFoo },
-        source => {
+        source  => {
             "foo.txt" => "abc",
             "bar.txt" => "def",
             "foo.tx"  => "ghi",
@@ -114,6 +78,20 @@ sub test_basic : Tests {
     );
 }
 
+sub test_filemode : Tests {
+    my $self     = shift;
+    my $root_dir = $self->create_dir( { "foo.txt" => "abc" } );
+    my $file     = $root_dir->child('foo.txt');
+    $file->chmod('0755');
+    my $ct = Code::TidyAll->new(
+        plugins  => { test_plugin('UpperText') => { select => '**/foo*' } },
+        root_dir => $root_dir,
+    );
+    $ct->process_paths($file);
+    is( $file->slurp,      "ABC" );
+    is( $file->stat->mode, 0100755 );
+}
+
 sub test_multiple_plugin_instances : Tests {
     my $self = shift;
     $self->tidy(
@@ -123,7 +101,7 @@ sub test_multiple_plugin_instances : Tests {
             %UpperText
         },
         source => { "foo.txt" => "abc", "foo.dat" => "def", "bar.txt" => "ghi" },
-        dest => {
+        dest   => {
             "foo.txt" => scalar( "ABC" x 6 ),
             "foo.dat" => scalar( "def" x 3 ),
             "bar.txt" => scalar( "GHI" x 2 )
@@ -157,10 +135,10 @@ sub test_plugin_order_and_atomicity : Tests {
 
     $self->tidy(
         plugins => { %AToZ, %ReverseFoo, %CheckUpper },
-        options => { verbose   => 1 },
-        source  => { "foo.txt" => "abc" },
-        dest    => { "foo.txt" => "abc" },
-        errors  => qr/lowercase found/,
+        options     => { verbose   => 1 },
+        source      => { "foo.txt" => "abc" },
+        dest        => { "foo.txt" => "abc" },
+        errors      => qr/lowercase found/,
         like_output => qr/foo.txt (.*ReverseFoo, .*CheckUpper)/
     );
 
@@ -202,13 +180,13 @@ sub test_iterations : Tests {
     my $self     = shift;
     my $root_dir = $self->create_dir( { "foo.txt" => "abc" } );
     my $ct       = Code::TidyAll->new(
-        plugins => { test_plugin('RepeatFoo') => { select => '**/foo*', times => 3 } },
+        plugins    => { test_plugin('RepeatFoo') => { select => '**/foo*', times => 3 } },
         root_dir   => $root_dir,
         iterations => 2
     );
-    my $file = "$root_dir/foo.txt";
+    my $file = $root_dir->child('foo.txt');
     $ct->process_paths($file);
-    is( read_file($file), scalar( "abc" x 9 ), "3^2 = 9" );
+    is( $file->slurp, scalar( "abc" x 9 ), "3^2 = 9" );
 }
 
 sub test_caching_and_backups : Tests {
@@ -240,13 +218,13 @@ sub test_caching_and_backups : Tests {
                         ( $chi        ? ( cache      => _chi() ) : () ),
                     );
                     my $output;
-                    my $file = "$root_dir/foo.txt";
-                    my $go   = sub {
+                    my $file = path( $root_dir, 'foo.txt' );
+                    my $go = sub {
                         $output = capture_stdout { $ct->process_paths($file) };
                     };
 
                     $go->();
-                    is( read_file($file), "ABC", "first file change $desc" );
+                    is( $file->slurp, "ABC", "first file change $desc" );
                     is( $output, "[tidied]  foo.txt\n", "first output $desc" );
 
                     $go->();
@@ -257,17 +235,17 @@ sub test_caching_and_backups : Tests {
                         is( $output, '', "second output $desc" );
                     }
 
-                    write_file( $file, "ABCD" );
+                    $file->spew("ABCD");
                     $go->();
                     is( $output, "[checked] foo.txt\n", "third output $desc" );
 
-                    write_file( $file, "def" );
+                    $file->spew("def");
                     $go->();
-                    is( read_file($file), "DEF", "fourth file change $desc" );
+                    is( $file->slurp, "DEF", "fourth file change $desc" );
                     is( $output, "[tidied]  foo.txt\n", "fourth output $desc" );
 
-                    my $backup_dir = $ct->data_dir . "/backups";
-                    mkpath( $backup_dir, 0, 0775 );
+                    my $backup_dir = $ct->data_dir->child('backups');
+                    $backup_dir->mkpath( { mode => 0775 } );
                     my @files;
                     find(
                         {
@@ -314,12 +292,15 @@ sub test_selects_and_ignores : Tests {
         root_dir => $root_dir,
         plugins  => {
             test_plugin('UpperText') => {
-                select => '**/*.pl **/*.pm b/bar.pm c/bar.pl',
-                ignore => 'a/foo.pl **/bar.pm c/baz.pl'
+                select => [qw( **/*.pl **/*.pm b/bar.pm c/bar.pl )],
+                ignore => [qw( a/foo.pl **/bar.pm c/baz.pl )],
             }
         }
     );
-    cmp_set( [ $ct->find_matched_files() ], [ "$root_dir/a/foo.pm", "$root_dir/b/foo.pl" ] );
+    cmp_set(
+        [ map { $_->stringify } $ct->find_matched_files() ],
+        [ "$root_dir/a/foo.pm", "$root_dir/b/foo.pl" ]
+    );
     cmp_deeply(
         [ map { $_->name } $ct->plugins_for_path("a/foo.pm") ],
         [ test_plugin('UpperText') ]
@@ -344,15 +325,15 @@ sub test_shebang : Tests {
         root_dir => $root_dir,
         plugins  => {
             test_plugin('UpperText') => {
-                select  => '**/*',
-                ignore  => '**/*.*',
-                shebang => 'perl perl5',
+                select  => ['**/*'],
+                ignore  => ['**/*.*'],
+                shebang => [ 'perl', 'perl5' ],
             }
         }
     );
     cmp_set(
-        [ $ct->find_matched_files() ],
-        [ map "$root_dir/$_", qw< a/foo b/bar b/baz > ],
+        [ map { $_->stringify } $ct->find_matched_files() ],
+        [ map {"$root_dir/$_"} qw( a/foo b/bar b/baz ) ],
     );
 }
 
@@ -366,7 +347,7 @@ sub test_dirs : Tests {
         my @results;
         my $output = capture_merged {
             my $ct = Code::TidyAll->new(
-                plugins => { %UpperText, %ReverseFoo },
+                plugins  => { %UpperText, %ReverseFoo },
                 root_dir => $root_dir,
                 ( $recursive ? ( recursive => 1 ) : () )
             );
@@ -377,10 +358,10 @@ sub test_dirs : Tests {
             is( scalar( grep { $_->state eq 'tidied' } @results ), 2, "2 tidied" );
             like( $output, qr/\[tidied\]  a\/foo.txt/ );
             like( $output, qr/\[tidied\]  a\/bar.txt/ );
-            is( read_file("$root_dir/a/foo.txt"), "IH" );
-            is( read_file("$root_dir/a/bar.txt"), "HI" );
-            is( read_file("$root_dir/a/bar.pl"),  "hi" );
-            is( read_file("$root_dir/b/foo.txt"), "hi" );
+            is( path( $root_dir, 'a', 'foo.txt' )->slurp, "IH" );
+            is( path( $root_dir, 'a', 'bar.txt' )->slurp, "HI" );
+            is( path( $root_dir, 'a', 'bar.pl' )->slurp,  "hi" );
+            is( path( $root_dir, 'b', 'foo.txt' )->slurp, "hi" );
         }
         else {
             is( @results,           1,       "1 result" );
@@ -390,6 +371,27 @@ sub test_dirs : Tests {
     }
 }
 
+sub test_paths_in_messages : Tests {
+    my $self = shift;
+
+    $self->tidy(
+        plugins => {
+            test_plugin('UpperText') => { select => '**/*.txt' },
+        },
+        options => { check_only => 1 },
+        source  => {
+            'path/to/file1.txt'        => "abc\n",
+            'path/to/longer/file2.txt' => "abc\n",
+            'top.txt'                  => "abc\n",
+        },
+        errors => [
+            qr{\[checked\] +path/to/file1\.txt},
+            qr{\[checked\] +path/to/longer/file2\.txt},
+            qr{\[checked\] +top\.txt},
+        ],
+    );
+}
+
 sub test_errors : Tests {
     my $self = shift;
 
@@ -410,7 +412,7 @@ sub test_errors : Tests {
     throws_ok {
         Code::TidyAll->new(
             root_dir => $root_dir,
-            plugins => { 'DoesNotExist' => { select => '**/*' } }
+            plugins  => { 'DoesNotExist' => { select => '**/*' } }
         )->plugin_objects;
     }
     qr/could not load plugin class/;
@@ -431,12 +433,35 @@ sub test_errors : Tests {
 
     $output = capture_stdout { $ct->process_paths("$root_dir/foo/bar.txt") };
     is( $output, "[tidied]  foo/bar.txt\n", "filename output" );
-    is( read_file("$root_dir/foo/bar.txt"), "ABC", "tidied" );
-    my $other_dir = realpath( tempdir_simple() );
-    write_file( "$other_dir/foo.txt", "ABC" );
+    is( path( $root_dir, 'foo', 'bar.txt' )->slurp, "ABC", "tidied" );
+    my $other_dir = tempdir_simple();
+    $other_dir->child('foo.txt')->spew("ABC");
     throws_ok { $ct->process_paths("$other_dir/foo.txt") } qr/not underneath root dir/;
 }
 
+sub test_git_files : Tests {
+    my $self = shift;
+
+    # Included examples:
+    # * basic modified file
+    # * renamed but not modified file (perltidy) -- the -> is in the filename
+    # * deleted file
+    # * renamed and modified file
+    my $status
+        = qq{## git-status-porcelain\0 M lib/Code/TidyAll/Git/Util.pm\0R  perltidyrc -> xyz\0perltidyrc\0 M t/lib/Test/Code/TidyAll/Basic.pm\0D  tidyall.ini\0RM weaver.initial\0weaver.ini\0};
+
+    require Code::TidyAll::Git::Util;
+    my @files = Code::TidyAll::Git::Util::_parse_status($status);
+    is_deeply(
+        \@files,
+        [
+            'lib/Code/TidyAll/Git/Util.pm',
+            't/lib/Test/Code/TidyAll/Basic.pm',
+            'weaver.initial',
+        ]
+    );
+}
+
 sub test_cli : Tests {
     my $self = shift;
     my $output;
@@ -458,12 +483,12 @@ sub test_cli : Tests {
             "conf at $conf_name",
             sub {
                 my $root_dir  = $self->create_dir();
-                my $conf_file = "$root_dir/$conf_name";
-                write_file( $conf_file, $cli_conf );
+                my $conf_file = $root_dir->child($conf_name);
+                $conf_file->spew($cli_conf);
 
-                write_file( "$root_dir/foo.txt", "hello" );
+                $root_dir->child('foo.txt')->spew("hello");
                 my $output = capture_stdout {
-                    $run->( "$root_dir/foo.txt", "-v" );
+                    $run->( $root_dir->child('foo.txt'), "-v" );
                 };
 
                 my ($params_msg)
@@ -481,24 +506,26 @@ sub test_cli : Tests {
                     'foo.txt'
                 );
                 is(
-                    read_file("$root_dir/foo.txt"), "HELLOHELLOHELLO",
+                    $root_dir->child('foo.txt')->slurp, "HELLOHELLOHELLO",
                     "tidied"
                 );
 
-                mkpath( "$root_dir/subdir", 0, 0775 );
-                write_file( "$root_dir/subdir/foo.txt",  "bye" );
-                write_file( "$root_dir/subdir/foo2.txt", "bye" );
-                my $cwd = realpath();
+                my $subdir = $root_dir->child('subdir');
+                $subdir->mkpath( { mode => 0775 } );
+                $subdir->child('foo.txt')->spew('bye');
+                $subdir->child('foo2.txt')->spew('bye');
+
+                my $cwd = cwd();
                 capture_stdout {
-                    my $dir = pushd "$root_dir/subdir";
+                    my $pushed = pushd($subdir);
                     system( $^X, "-I$cwd/lib", "-I$cwd/t/lib", "$cwd/bin/tidyall", 'foo.txt' );
                 };
                 is(
-                    read_file("$root_dir/subdir/foo.txt"), "BYEBYEBYE",
+                    $root_dir->child( 'subdir', 'foo.txt' )->slurp, "BYEBYEBYE",
                     "foo.txt tidied"
                 );
                 is(
-                    read_file("$root_dir/subdir/foo2.txt"), "bye",
+                    $root_dir->child( 'subdir', 'foo2.txt' )->slurp, "bye",
                     "foo2.txt not tidied"
                 );
 
@@ -507,7 +534,7 @@ sub test_cli : Tests {
                 my ( $stdout, $stderr ) = capture {
                     open(
                         my $fh, "|-", @cmd, "-p",
-                        "$root_dir/does_not_exist/foo.txt"
+                        $root_dir->child(qw( does_not_exist foo.txt ))
                     );
                     print $fh "echo";
                 };
@@ -517,7 +544,7 @@ sub test_cli : Tests {
                 # -p / --pipe error
                 #
                 ( $stdout, $stderr ) = capture {
-                    open( my $fh, "|-", @cmd, "--pipe", "$root_dir/foo.txt" );
+                    open( my $fh, "|-", @cmd, "--pipe", $root_dir->child('foo.txt') );
                     print $fh "abc1";
                 };
                 is( $stdout, "abc1", "pipe: stdin mirrored to stdout" );
@@ -527,16 +554,4 @@ sub test_cli : Tests {
     }
 }
 
-$cli_conf = '
-backup_ttl = 15m
-verbose = 1
-
-[+Code::TidyAll::Test::Plugin::UpperText]
-select = **/*.txt
-
-[+Code::TidyAll::Test::Plugin::RepeatFoo]
-select = **/foo*
-times = 3
-';
-
 1;
